"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = require("events");
class CloudOperationBase extends events_1.EventEmitter {
    constructor(id, serverResponse, $logger, $nsCloudOutputFilter, $nsCloudS3Helper) {
        super();
        this.id = id;
        this.serverResponse = serverResponse;
        this.$logger = $logger;
        this.$nsCloudOutputFilter = $nsCloudOutputFilter;
        this.$nsCloudS3Helper = $nsCloudS3Helper;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.initCore();
                this.initialized = true;
            }
            catch (err) {
                throw err;
            }
        });
    }
    waitForResult() {
        return __awaiter(this, void 0, void 0, function* () {
            this.isInitialized();
            const result = yield this.waitForResultCore();
            return result;
        });
    }
    cleanup(exitCode) {
        return __awaiter(this, void 0, void 0, function* () {
            this.removeAllListeners();
        });
    }
    getResult() {
        this.isInitialized();
        return this.result;
    }
    getCollectedLogs() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return this.$nsCloudOutputFilter.filter(yield this.$nsCloudS3Helper.getContentOfS3File(this.serverResponse.outputUrl));
            }
            catch (err) {
                this.$logger.warn(`Unable to get cloud operation output. Error is: ${err}`);
            }
            return "";
        });
    }
    isInitialized() {
        if (!this.initialized) {
            throw new Error("Not initialized");
        }
    }
}
CloudOperationBase.OPERATION_COMPLETE_STATUS = "Success";
CloudOperationBase.OPERATION_FAILED_STATUS = "Failed";
CloudOperationBase.OPERATION_IN_PROGRESS_STATUS = "InProgress";
exports.CloudOperationBase = CloudOperationBase;
