"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const promiseRetry = require("promise-retry");
const constants_1 = require("../constants");
const cloud_operation_base_1 = require("./cloud-operation-base");
class CloudOperationV1 extends cloud_operation_base_1.CloudOperationBase {
    constructor(id, serverResponse, $logger, $nsCloudOutputFilter, $nsCloudS3Helper) {
        super(id, serverResponse, $logger, $nsCloudOutputFilter, $nsCloudS3Helper);
        this.id = id;
        this.serverResponse = serverResponse;
        this.$logger = $logger;
        this.$nsCloudOutputFilter = $nsCloudOutputFilter;
        this.$nsCloudS3Helper = $nsCloudS3Helper;
        this.outputCursorPosition = 0;
    }
    sendMessage(message) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("This version of the cloud operation does not support sending messages to the cloud services.");
        });
    }
    cleanup(exitCode) {
        const _super = Object.create(null, {
            cleanup: { get: () => super.cleanup }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.cleanup.call(this, exitCode);
            clearInterval(this.statusCheckInterval);
            clearInterval(this.logsCheckInterval);
        });
    }
    initCore() {
        return __awaiter(this, void 0, void 0, function* () {
            const promiseRetryOptions = {
                retries: CloudOperationV1.OPERATION_STATUS_CHECK_RETRY_COUNT,
                minTimeout: CloudOperationV1.OPERATION_STATUS_CHECK_INTERVAL
            };
            yield promiseRetry((retry, attempt) => __awaiter(this, void 0, void 0, function* () {
                try {
                    this.serverStatus = yield this.$nsCloudS3Helper.getJsonObjectFromS3File(this.serverResponse.statusUrl);
                    return this.serverStatus;
                }
                catch (err) {
                    this.$logger.trace(err);
                    retry(new Error("Failed to start."));
                }
            }), promiseRetryOptions);
            this.pollForLogs();
        });
    }
    waitForResultCore() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                this.statusCheckInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
                    const status = this.serverStatus && this.serverStatus.status;
                    if (status === CloudOperationV1.OPERATION_IN_PROGRESS_STATUS) {
                        this.serverStatus = yield this.$nsCloudS3Helper.getJsonObjectFromS3File(this.serverResponse.statusUrl);
                    }
                    if (!this.hasLogPollCompleted) {
                        return;
                    }
                    if (this.serverStatus.status === CloudOperationV1.OPERATION_COMPLETE_STATUS) {
                        clearInterval(this.statusCheckInterval);
                        this.result = yield this.$nsCloudS3Helper.getJsonObjectFromS3File(this.serverResponse.resultUrl);
                        return resolve(this.result);
                    }
                    if (this.serverStatus.status === CloudOperationV1.OPERATION_FAILED_STATUS) {
                        try {
                            this.result = yield this.$nsCloudS3Helper.getJsonObjectFromS3File(this.serverResponse.resultUrl);
                            clearInterval(this.statusCheckInterval);
                            resolve(this.result);
                        }
                        catch (err) {
                            clearInterval(this.statusCheckInterval);
                            this.$logger.trace(err);
                            return reject(new Error("Cloud operation failed"));
                        }
                    }
                }), CloudOperationV1.OPERATION_STATUS_CHECK_INTERVAL);
            });
        });
    }
    pollForLogs() {
        this.logsCheckInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
            if (this.snoozeLogPoll) {
                return;
            }
            const hasCompleted = this.serverStatus.status !== cloud_operation_base_1.CloudOperationBase.OPERATION_IN_PROGRESS_STATUS;
            if (hasCompleted) {
                clearInterval(this.logsCheckInterval);
            }
            this.snoozeLogPoll = true;
            yield this.getCloudOperationLogs();
            this.snoozeLogPoll = false;
            this.hasLogPollCompleted = hasCompleted;
        }), CloudOperationV1.OPERATION_STATUS_CHECK_INTERVAL);
    }
    getCloudOperationLogs() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const logs = yield this.$nsCloudS3Helper.getContentOfS3File(this.serverResponse.outputUrl);
                const contentToLog = this.$nsCloudOutputFilter.filter(logs.substr(this.outputCursorPosition));
                if (contentToLog) {
                    const data = {
                        type: constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_OUTPUT,
                        cloudOperationId: this.id,
                        body: {
                            data: contentToLog + os_1.EOL,
                            pipe: "stdout"
                        }
                    };
                    this.emit(constants_1.CloudCommunicationEvents.MESSAGE, data);
                }
                this.outputCursorPosition = logs.length <= 0 ? 0 : logs.length - 1;
            }
            catch (err) {
                this.$logger.trace(`Error while getting server logs: ${err}`);
            }
        });
    }
}
CloudOperationV1.OPERATION_STATUS_CHECK_RETRY_COUNT = 8;
CloudOperationV1.OPERATION_STATUS_CHECK_INTERVAL = 1500;
module.exports = CloudOperationV1;
