"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cloud_operation_base_1 = require("./cloud-operation-base");
const constants_1 = require("../constants");
class CloudOperationV2 extends cloud_operation_base_1.CloudOperationBase {
    constructor(id, serverResponse, $logger, $nsCloudOutputFilter, $nsCloudS3Helper, $nsCloudCommunicationChannelFactory) {
        super(id, serverResponse, $logger, $nsCloudOutputFilter, $nsCloudS3Helper);
        this.id = id;
        this.serverResponse = serverResponse;
        this.$logger = $logger;
        this.$nsCloudOutputFilter = $nsCloudOutputFilter;
        this.$nsCloudS3Helper = $nsCloudS3Helper;
        this.$nsCloudCommunicationChannelFactory = $nsCloudCommunicationChannelFactory;
        this.communicationChannel = this.$nsCloudCommunicationChannelFactory.create(serverResponse.communicationChannel, this.id);
    }
    sendMessage(message) {
        return __awaiter(this, void 0, void 0, function* () {
            this.isInitialized();
            yield this.communicationChannel.sendMessage(message);
        });
    }
    cleanup(exitCode) {
        const _super = Object.create(null, {
            cleanup: { get: () => super.cleanup }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield _super.cleanup.call(this);
            if (this.waitToStartTimeout) {
                clearTimeout(this.waitToStartTimeout);
                this.waitToStartTimeout = null;
            }
            this.communicationChannel.removeAllListeners();
            yield this.communicationChannel.close(exitCode);
        });
    }
    initCore() {
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            this.waitToStartTimeout = setTimeout(() => {
                clearTimeout(this.waitToStartTimeout);
                this.waitToStartTimeout = null;
                if (!this.status) {
                    return reject(new Error("Communication channel init timed out."));
                }
            }, CloudOperationV2.WAIT_TO_START_TIMEOUT);
            const closeHandler = (code) => {
                reject(new Error(`Communication channel closed with code ${code}`));
            };
            try {
                this.communicationChannel.once(constants_1.CloudCommunicationEvents.CLOSE, closeHandler);
                this.communicationChannel.on(constants_1.CloudCommunicationEvents.MESSAGE, (m) => this.emit(constants_1.CloudCommunicationEvents.MESSAGE, m));
                yield this.communicationChannel.connect();
                this.status = CloudOperationV2.OPERATION_IN_PROGRESS_STATUS;
                this.waitResultPromise = this.getResultPromise();
                this.communicationChannel.removeListener(constants_1.CloudCommunicationEvents.CLOSE, closeHandler);
                resolve();
            }
            catch (err) {
                return reject(err);
            }
        }));
    }
    waitForResultCore() {
        return this.waitResultPromise;
    }
    getResultPromise() {
        return new Promise((resolve, reject) => {
            this.communicationChannel.once(constants_1.CloudCommunicationEvents.CLOSE, code => {
                if (this.status && (this.status !== cloud_operation_base_1.CloudOperationBase.OPERATION_COMPLETE_STATUS && this.status !== cloud_operation_base_1.CloudOperationBase.OPERATION_FAILED_STATUS)) {
                    reject(new Error(`Communication channel closed with code ${code}`));
                }
            });
            this.communicationChannel.on(constants_1.CloudCommunicationEvents.MESSAGE, (m) => __awaiter(this, void 0, void 0, function* () {
                if (m.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_RESULT) {
                    this.result = m.body;
                    if (this.result.code === 0 || this.result.data) {
                        this.status = CloudOperationV2.OPERATION_COMPLETE_STATUS;
                    }
                    else {
                        this.status = CloudOperationV2.OPERATION_FAILED_STATUS;
                    }
                    resolve(this.result);
                }
            }));
        });
    }
}
CloudOperationV2.WAIT_TO_START_TIMEOUT = 10 * 60 * 1000;
module.exports = CloudOperationV2;
