"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
const communication_channel_base_1 = require("./communication-channel-base");
const constants_1 = require("../../constants");
class WebSocketCommunicationChannel extends communication_channel_base_1.CommunicationChannelBase {
    constructor(cloudOperationId, data, $logger, $nsCloudWebSocketFactory) {
        super(cloudOperationId, data, $logger);
        this.cloudOperationId = cloudOperationId;
        this.data = data;
        this.$logger = $logger;
        this.$nsCloudWebSocketFactory = $nsCloudWebSocketFactory;
    }
    closeCore(code, reason) {
        return __awaiter(this, void 0, void 0, function* () {
            this.ws.removeAllListeners();
            if (this.ws.readyState === this.ws.OPEN) {
                this.ws.close();
            }
        });
    }
    sendMessageCore(message) {
        return new Promise((resolve, reject) => {
            try {
                const id = uuid_1.v4();
                const wsMsg = {
                    id,
                    action: constants_1.CloudOperationWebSocketMessageActions.SEND_MESSAGE,
                    cloudOperationId: this.cloudOperationId,
                    body: message
                };
                this.$logger.trace(wsMsg);
                this.ws.send(JSON.stringify(wsMsg), err => {
                    if (err) {
                        return reject(err);
                    }
                    resolve(id);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    connectCore() {
        return __awaiter(this, void 0, void 0, function* () {
            this.ws = this.$nsCloudWebSocketFactory.create(this.data.config.url);
            yield new Promise((resolve, reject) => {
                const closeHandler = (c, r) => {
                    reject(new Error(`Connection closed with code: ${c}`));
                    this.close(c, r);
                };
                const unexpectedResponseHandler = () => {
                    const errMsg = "Unexpected response received.";
                    reject(new Error(errMsg));
                    this.close(constants_1.CloudCommunicationChannelExitCodes.UNEXPECTED_RESPONSE, errMsg);
                };
                this.ws.once("close", closeHandler);
                this.ws.once("unexpected-response", unexpectedResponseHandler);
                this.ws.once("open", () => {
                    this.ws.removeListener("close", closeHandler);
                    this.ws.removeListener("unexpected-response", unexpectedResponseHandler);
                    this.addChannelListeners();
                    resolve();
                });
            });
        });
    }
    addChannelListeners() {
        this.ws.on(constants_1.CloudCommunicationEvents.MESSAGE, m => super.handleMessage(m));
        this.ws.on(constants_1.CloudCommunicationEvents.ERROR, err => this.emit("error", err));
        this.ws.once(constants_1.CloudCommunicationEvents.CLOSE, this.close.bind(this));
    }
}
exports.WebSocketCommunicationChannel = WebSocketCommunicationChannel;
