"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = require("os");
const account_command_base_1 = require("./account-command-base");
const constants_1 = require("../../constants");
const helpers_1 = require("../../helpers");
class UsageCommand extends account_command_base_1.AccountCommandBase {
    constructor($nsCloudErrorsService, $nsCloudUserService, $nsCloudAccountsService, $nsCloudEulaCommandHelper, $nsCloudOptionsProvider, $options, $logger) {
        super($nsCloudErrorsService, $nsCloudUserService);
        this.$nsCloudAccountsService = $nsCloudAccountsService;
        this.$nsCloudEulaCommandHelper = $nsCloudEulaCommandHelper;
        this.$nsCloudOptionsProvider = $nsCloudOptionsProvider;
        this.$options = $options;
        this.$logger = $logger;
        this.allowedParameters = [];
    }
    get dashedOptions() {
        return this.$nsCloudOptionsProvider.dashedOptions;
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            yield this.$nsCloudEulaCommandHelper.ensureEulaIsAccepted();
            const usage = yield this.$nsCloudAccountsService.getUsageInfo(this.$options.accountId);
            let output;
            const groupedUsage = _(usage)
                .map(u => {
                const result = {
                    feature: u.feature,
                    allowedUsage: u.allowedUsage,
                    performed: u.usage,
                    remaining: u.unlimited ? u.allowedUsage : u.allowedUsage - u.usage,
                    unlimited: !!u.unlimited,
                    editionType: u.editionType,
                    licenseExpiration: u.licenseExpiration,
                    licenseExpirationDate: u.licenseExpirationDate,
                    resetDate: u.resetDate,
                    licenseType: u.licenseType
                };
                return result;
            })
                .groupBy(u => u.feature)
                .value();
            if (this.$options.json) {
                output = helpers_1.stringifyWithIndentation(groupedUsage);
            }
            else {
                const tables = _.map(groupedUsage, (g, feature) => {
                    return helpers_1.createTable([`${feature} performed`, `${feature} remaining`, "Reset Date", "License Expiration", "License Type", "Edition Type"], g.map(u => {
                        const result = [u.performed.toString()];
                        if (u.unlimited) {
                            result.push(constants_1.UNLIMITED);
                        }
                        else {
                            const remainingUsage = u.allowedUsage - u.performed;
                            result.push(remainingUsage.toString());
                        }
                        return result.concat(this.toPrettyDate(u.resetDate), this.toPrettyDate(u.licenseExpirationDate), u.licenseType, u.editionType);
                    }));
                });
                output = tables.join(os_1.EOL);
            }
            this.$logger.info(output);
        });
    }
    toPrettyDate(date) {
        return new Date(date).toDateString();
    }
}
exports.UsageCommand = UsageCommand;
$injector.registerCommand("account|usage", UsageCommand);
