"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../../helpers");
const interactive_cloud_command_1 = require("../interactive-cloud-command");
class CleanCloudWorkspace extends interactive_cloud_command_1.InteractiveCloudCommand {
    constructor($nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter, $nsCloudEulaCommandHelper, $nsCloudProjectService, $projectData) {
        super($nsCloudProjectService, $nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter);
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$nsCloudEulaCommandHelper = $nsCloudEulaCommandHelper;
        this.$nsCloudProjectService = $nsCloudProjectService;
        this.$projectData = $projectData;
        this.allowedParameters = [];
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!args || args.length > 2) {
                return false;
            }
            yield this.$nsCloudEulaCommandHelper.ensureEulaIsAccepted();
            return true;
        });
    }
    executeCore(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let appIdentifier;
            let projectName;
            if (args.length === 0) {
                try {
                    this.$projectData.initializeProjectData();
                    appIdentifier = this.$projectData.projectIdentifiers || this.$projectData.projectId;
                    projectName = this.$projectData.projectName;
                }
                catch (err) {
                    appIdentifier = yield this.promptForAppId();
                    projectName = yield this.promptForProjectName();
                }
            }
            else if (args.length === 1) {
                appIdentifier = yield this.getParameterValue(args[0], () => this.promptForAppId());
                projectName = yield this.promptForProjectName();
            }
            else {
                appIdentifier = yield this.getParameterValue(args[0], () => this.promptForAppId());
                projectName = yield this.getParameterValue(args[1], () => this.promptForProjectName());
            }
            yield this.$nsCloudProjectService.cleanupProject({ appIdentifier, projectName });
        });
    }
    promptForProjectName() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!helpers_1.isInteractive()) {
                this.$nsCloudErrorsService.fail(CleanCloudWorkspace.COMMAND_REQUIREMENTS_ERROR_MESSAGE);
            }
            return this.$prompter.getString("Project name:", { allowEmpty: false });
        });
    }
    promptForAppId() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!helpers_1.isInteractive()) {
                this.$nsCloudErrorsService.fail(CleanCloudWorkspace.COMMAND_REQUIREMENTS_ERROR_MESSAGE);
            }
            return this.$prompter.getString("App Id:", { allowEmpty: false });
        });
    }
    getParameterValue(param, action) {
        return __awaiter(this, void 0, void 0, function* () {
            if (param.trim().length > 0) {
                return param;
            }
            return action();
        });
    }
}
CleanCloudWorkspace.COMMAND_REQUIREMENTS_ERROR_MESSAGE = "The command should be executed inside project or the app id and project name parameters must be provided.";
exports.CleanCloudWorkspace = CleanCloudWorkspace;
$injector.registerCommand(["cloud|clean|workspace"], CleanCloudWorkspace);
