"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
const constants_1 = require("../constants");
const interactive_cloud_command_1 = require("./interactive-cloud-command");
class CloudCodesignCommand extends interactive_cloud_command_1.InteractiveCloudCommand {
    constructor($nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter, $nsCloudEulaCommandHelper, $devicePlatformsConstants, $devicesService, $nsCloudOptionsProvider, $options, $projectData, $nsCloudCodesignService) {
        super($nsCloudCodesignService, $nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter);
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$nsCloudEulaCommandHelper = $nsCloudEulaCommandHelper;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$devicesService = $devicesService;
        this.$nsCloudOptionsProvider = $nsCloudOptionsProvider;
        this.$options = $options;
        this.$projectData = $projectData;
        this.$nsCloudCodesignService = $nsCloudCodesignService;
        this.platform = this.$devicePlatformsConstants.iOS;
        this.$projectData.initializeProjectData();
    }
    get dashedOptions() {
        return this.$nsCloudOptionsProvider.dashedOptions;
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            yield this.$nsCloudEulaCommandHelper.ensureEulaIsAccepted();
            if (args.length > 2 || (!helpers_1.isInteractive() && args.length < 2)) {
                this.$nsCloudErrorsService.failWithHelp(constants_1.ERROR_MESSAGES.COMMAND_REQUIRES_APPLE_USERNAME_PASS);
            }
            yield this.$devicesService.initialize({ shouldReturnImmediateResult: false, platform: this.platform, skipEmulatorStart: true });
            this.devices = this.$devicesService.getDeviceInstances()
                .filter(d => !d.isEmulator && d.deviceInfo.platform.toLowerCase() === this.platform.toLowerCase())
                .map(d => d.deviceInfo);
            if (!this.devices || this.devices.length === 0) {
                this.$nsCloudErrorsService.failWithHelp("Please attach iOS devices for which to generate codesign files.");
            }
            return true;
        });
    }
    executeCore(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let username = args[0];
            let password = args[1];
            if (!username) {
                username = yield this.$prompter.getString("Apple ID", { allowEmpty: false });
            }
            if (!password) {
                password = yield this.$prompter.getPassword("Apple ID password");
            }
            this.$logger.info("Generating codesign files in the cloud");
            const codesignData = {
                username, password,
                clean: true,
                sharedCloud: this.$options.sharedCloud,
                platform: this.platform,
                attachedDevices: this.devices
            };
            yield this.$nsCloudCodesignService.generateCodesignFiles(codesignData, this.$projectData.projectDir);
        });
    }
}
exports.CloudCodesignCommand = CloudCodesignCommand;
$injector.registerCommand("codesign|cloud", CloudCodesignCommand);
