"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants");
const helpers_1 = require("../helpers");
const interactive_cloud_command_1 = require("./interactive-cloud-command");
class CloudDevAppleLogin extends interactive_cloud_command_1.InteractiveCloudCommand {
    constructor($nsCloudProcessService, $nsCloudEulaCommandHelper, $nsCloudOptionsProvider, $options, $nsCloudErrorsService, $fs, $logger, $prompter, $nsCloudAppleService, $nsCloudBuildCommandHelper) {
        super($nsCloudAppleService, $nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter);
        this.$nsCloudEulaCommandHelper = $nsCloudEulaCommandHelper;
        this.$nsCloudOptionsProvider = $nsCloudOptionsProvider;
        this.$options = $options;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$nsCloudAppleService = $nsCloudAppleService;
        this.$nsCloudBuildCommandHelper = $nsCloudBuildCommandHelper;
    }
    get dashedOptions() {
        return this.$nsCloudOptionsProvider.dashedOptions;
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            yield this.$nsCloudEulaCommandHelper.ensureEulaIsAccepted();
            if (args.length > 2 || (!helpers_1.isInteractive() && args.length < 1)) {
                this.$nsCloudErrorsService.failWithHelp(constants_1.ERROR_MESSAGES.COMMAND_REQUIRES_APPLE_USERNAME_PASS);
            }
            return true;
        });
    }
    executeCore(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const otp = this.$options.otp;
            if (otp) {
                this.predefinedAnswers.push({ searchString: "6 digit code", answer: otp }, { searchString: "4 digit code", answer: otp });
            }
            const credentials = yield this.$nsCloudBuildCommandHelper.getAppleCredentials(args);
            const result = yield this.$nsCloudAppleService.appleLogin(credentials);
            const outputPath = this.$options.outputPath;
            if (outputPath) {
                this.$fs.writeFile(outputPath, result);
                this.$logger.info(`Apple session saved in ${outputPath}`);
            }
            else {
                this.$logger.info(result);
            }
        });
    }
}
exports.CloudDevAppleLogin = CloudDevAppleLogin;
$injector.registerCommand("cloud|dev-apple-login", CloudDevAppleLogin);
