"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants");
const helpers_1 = require("../helpers");
const interactive_cloud_command_1 = require("./interactive-cloud-command");
class CloudPublish extends interactive_cloud_command_1.InteractiveCloudCommand {
    constructor($nsCloudProcessService, $nsCloudOptionsProvider, $nsCloudErrorsService, $logger, $prompter, $projectData, $options, $devicePlatformsConstants, $nsCloudAndroidBundleValidatorHelper, $nsCloudPublishService) {
        super($nsCloudPublishService, $nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter);
        this.$nsCloudOptionsProvider = $nsCloudOptionsProvider;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$projectData = $projectData;
        this.$options = $options;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$nsCloudAndroidBundleValidatorHelper = $nsCloudAndroidBundleValidatorHelper;
        this.$nsCloudPublishService = $nsCloudPublishService;
        this.$projectData.initializeProjectData();
    }
    get dashedOptions() {
        return this.$nsCloudOptionsProvider.dashedOptions;
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            this.$nsCloudAndroidBundleValidatorHelper.validateNoAab();
            return true;
        });
    }
}
class CloudPublishAndroid extends CloudPublish {
    constructor($nsCloudOptionsProvider, $logger, $nsCloudProcessService, $nsCloudBuildCommandHelper, $nsCloudEulaCommandHelper, $nsCloudErrorsService, $nsCloudPublishService, $prompter, $projectData, $options, $devicePlatformsConstants, $nsCloudAndroidBundleValidatorHelper) {
        super($nsCloudProcessService, $nsCloudOptionsProvider, $nsCloudErrorsService, $logger, $prompter, $projectData, $options, $devicePlatformsConstants, $nsCloudAndroidBundleValidatorHelper, $nsCloudPublishService);
        this.$nsCloudBuildCommandHelper = $nsCloudBuildCommandHelper;
        this.$nsCloudEulaCommandHelper = $nsCloudEulaCommandHelper;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$nsCloudPublishService = $nsCloudPublishService;
        this.$prompter = $prompter;
        this.$projectData = $projectData;
        this.$options = $options;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$nsCloudAndroidBundleValidatorHelper = $nsCloudAndroidBundleValidatorHelper;
    }
    canExecute(args) {
        const _super = Object.create(null, {
            canExecute: { get: () => super.canExecute }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield this.$nsCloudEulaCommandHelper.ensureEulaIsAccepted();
            yield _super.canExecute.call(this, args);
            if (args.length > 1 || (!helpers_1.isInteractive() && args.length < 1)) {
                this.$nsCloudErrorsService.failWithHelp("The command accepts only one parameter - Path to authentication JSON");
            }
            return true;
        });
    }
    executeCore(args) {
        return __awaiter(this, void 0, void 0, function* () {
            let pathToAuthJson = args[0];
            let track = this.$options.track;
            if (!pathToAuthJson) {
                pathToAuthJson = yield this.$prompter.getString("Path to auth JSON", { allowEmpty: false });
            }
            if (!track) {
                track = yield this.$prompter.getString("Track", { defaultAction: () => constants_1.DEFAULT_ANDROID_PUBLISH_TRACK });
            }
            const packagePath = yield this.$nsCloudBuildCommandHelper.buildForPublishingPlatform(this.$devicePlatformsConstants.Android);
            return this.$nsCloudPublishService.publishToGooglePlay({
                track,
                pathToAuthJson,
                packagePaths: [packagePath],
                sharedCloud: this.$options.sharedCloud,
                projectDir: this.$projectData.projectDir,
                androidReleaseStatus: this.$options.androidReleaseStatus
            });
        });
    }
}
exports.CloudPublishAndroid = CloudPublishAndroid;
$injector.registerCommand("cloud|publish|android", CloudPublishAndroid);
class CloudPublishIos extends CloudPublish {
    constructor($nsCloudOptionsProvider, $logger, $nsCloudProcessService, $nsCloudBuildCommandHelper, $nsCloudEulaCommandHelper, $nsCloudErrorsService, $nsCloudPublishService, $prompter, $projectData, $options, $devicePlatformsConstants, $nsCloudAndroidBundleValidatorHelper) {
        super($nsCloudProcessService, $nsCloudOptionsProvider, $nsCloudErrorsService, $logger, $prompter, $projectData, $options, $devicePlatformsConstants, $nsCloudAndroidBundleValidatorHelper, $nsCloudPublishService);
        this.$nsCloudBuildCommandHelper = $nsCloudBuildCommandHelper;
        this.$nsCloudEulaCommandHelper = $nsCloudEulaCommandHelper;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$nsCloudPublishService = $nsCloudPublishService;
        this.$prompter = $prompter;
        this.$projectData = $projectData;
        this.$options = $options;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
    }
    canExecute(args) {
        const _super = Object.create(null, {
            canExecute: { get: () => super.canExecute }
        });
        return __awaiter(this, void 0, void 0, function* () {
            yield this.$nsCloudEulaCommandHelper.ensureEulaIsAccepted();
            yield _super.canExecute.call(this, args);
            if (args.length > 2 || (!helpers_1.isInteractive() && args.length < 1)) {
                this.$nsCloudErrorsService.failWithHelp(constants_1.ERROR_MESSAGES.COMMAND_REQUIRES_APPLE_USERNAME_PASS);
            }
            return true;
        });
    }
    executeCore(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const credentials = yield this.$nsCloudBuildCommandHelper.getExtendedAppleCredentials(args, this.$options);
            const packagePath = yield this.$nsCloudBuildCommandHelper.buildForPublishingPlatform(this.$devicePlatformsConstants.iOS);
            const itunesPublishData = {
                credentials,
                packagePaths: [packagePath],
                sharedCloud: this.$options.sharedCloud,
                projectDir: this.$projectData.projectDir
            };
            yield this.$nsCloudPublishService.publishToItunesConnect(itunesPublishData);
        });
    }
}
exports.CloudPublishIos = CloudPublishIos;
$injector.registerCommand("cloud|publish|ios", CloudPublishIos);
