"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const interactive_cloud_command_1 = require("./interactive-cloud-command");
class CloudRunCommand extends interactive_cloud_command_1.InteractiveCloudCommand {
    constructor($nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter, $liveSyncCommandHelper, $nsCloudEulaCommandHelper, $nsCloudBuildService, $nsCloudBuildCommandHelper, $nsCloudOptionsProvider, $nsCloudPolyfillService, $options, $projectData, $nsCloudAndroidBundleValidatorHelper) {
        super($nsCloudBuildService, $nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter);
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$liveSyncCommandHelper = $liveSyncCommandHelper;
        this.$nsCloudEulaCommandHelper = $nsCloudEulaCommandHelper;
        this.$nsCloudBuildService = $nsCloudBuildService;
        this.$nsCloudBuildCommandHelper = $nsCloudBuildCommandHelper;
        this.$nsCloudOptionsProvider = $nsCloudOptionsProvider;
        this.$nsCloudPolyfillService = $nsCloudPolyfillService;
        this.$options = $options;
        this.$projectData = $projectData;
        this.$nsCloudAndroidBundleValidatorHelper = $nsCloudAndroidBundleValidatorHelper;
        this.$projectData.initializeProjectData();
        if (!this.$options.justlaunch) {
            const cleanupService = this.$nsCloudPolyfillService.getPolyfillObject("cleanupService", { setShouldDispose: () => { } });
            cleanupService.setShouldDispose(true);
        }
    }
    get dashedOptions() {
        return this.$nsCloudOptionsProvider.dashedOptions;
    }
    canExecute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            yield this.$nsCloudEulaCommandHelper.ensureEulaIsAccepted();
            this.$nsCloudAndroidBundleValidatorHelper.validateNoAab();
            if (args.length) {
                this.$nsCloudErrorsService.failWithHelp("This input is not valid for the cloud run command");
            }
            return true;
        });
    }
    executeCore(args) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.$liveSyncCommandHelper.executeCommandLiveSync(this.platform, {
                getOutputDirectory: this.$nsCloudBuildService.getServerOperationOutputDirectory.bind(this.$nsCloudBuildService),
                buildPlatform: this.$nsCloudBuildCommandHelper.buildPlatform.bind(this.$nsCloudBuildCommandHelper),
                skipNativePrepare: true
            });
        });
    }
}
exports.CloudRunCommand = CloudRunCommand;
$injector.registerCommand(["run|cloud|*all", "cloud|run|*all"], CloudRunCommand);
class CloudRunIosCommand extends CloudRunCommand {
    constructor($liveSyncCommandHelper, $nsCloudEulaCommandHelper, $nsCloudBuildCommandHelper, $nsCloudBuildService, $nsCloudOptionsProvider, $nsCloudPolyfillService, $nsCloudProcessService, $options, $projectData, $nsCloudAndroidBundleValidatorHelper, $nsCloudErrorsService, $logger, $prompter, $devicePlatformsConstants) {
        super($nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter, $liveSyncCommandHelper, $nsCloudEulaCommandHelper, $nsCloudBuildService, $nsCloudBuildCommandHelper, $nsCloudOptionsProvider, $nsCloudPolyfillService, $options, $projectData, $nsCloudAndroidBundleValidatorHelper);
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.allowedParameters = [];
    }
    get platform() {
        return this.$devicePlatformsConstants.iOS;
    }
}
exports.CloudRunIosCommand = CloudRunIosCommand;
$injector.registerCommand(["run|cloud|ios", "cloud|run|ios"], CloudRunIosCommand);
class CloudRunAndroidCommand extends CloudRunCommand {
    constructor($liveSyncCommandHelper, $nsCloudEulaCommandHelper, $nsCloudBuildCommandHelper, $nsCloudBuildService, $nsCloudOptionsProvider, $nsCloudPolyfillService, $nsCloudProcessService, $options, $projectData, $nsCloudAndroidBundleValidatorHelper, $nsCloudErrorsService, $logger, $prompter, $devicePlatformsConstants) {
        super($nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter, $liveSyncCommandHelper, $nsCloudEulaCommandHelper, $nsCloudBuildService, $nsCloudBuildCommandHelper, $nsCloudOptionsProvider, $nsCloudPolyfillService, $options, $projectData, $nsCloudAndroidBundleValidatorHelper);
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.allowedParameters = [];
    }
    get platform() {
        return this.$devicePlatformsConstants.Android;
    }
}
exports.CloudRunAndroidCommand = CloudRunAndroidCommand;
$injector.registerCommand(["run|cloud|android", "cloud|run|android"], CloudRunAndroidCommand);
