"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants");
const helpers_1 = require("../helpers");
class EulaCommandHelper {
    constructor($nsCloudErrorsService, $logger, $nsCloudEulaService, $prompter) {
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$nsCloudEulaService = $nsCloudEulaService;
        this.$prompter = $prompter;
    }
    acceptEula() {
        this.$logger.printMarkdown(`Accepting EULA located at ${constants_1.EulaConstants.eulaUrl}.`);
        return this.$nsCloudEulaService.acceptEula();
    }
    ensureEulaIsAccepted() {
        return __awaiter(this, void 0, void 0, function* () {
            const eulaData = yield this.$nsCloudEulaService.getEulaDataWithCache();
            if (!eulaData.shouldAcceptEula) {
                this.$logger.trace("Ensure EULA accepted: no need to accept EULA - already accepted.");
                return;
            }
            this.$logger.printMarkdown(`In order to use cloud services, you must accept our EULA. You can read it here: ${constants_1.EulaConstants.eulaUrl}.`);
            const actionOnError = () => this.$nsCloudErrorsService.fail("You cannot use cloud services without accepting the EULA.");
            if (!helpers_1.isInteractive()) {
                return actionOnError();
            }
            const hasAcceptedEula = yield this.$prompter.confirm("Do you accept the EULA?");
            if (!hasAcceptedEula) {
                return actionOnError();
            }
            yield this.$nsCloudEulaService.acceptEula();
        });
    }
}
exports.EulaCommandHelper = EulaCommandHelper;
$injector.register("nsCloudEulaCommandHelper", EulaCommandHelper);
