"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
const constants_1 = require("../constants");
class InteractiveCloudCommand {
    constructor(interactiveService, $nsCloudProcessService, $nsCloudErrorsService, $logger, $prompter) {
        this.interactiveService = interactiveService;
        this.$nsCloudProcessService = $nsCloudProcessService;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$logger = $logger;
        this.$prompter = $prompter;
        this.predefinedAnswers = [];
    }
    execute(args) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const messageHandler = (msg) => __awaiter(this, void 0, void 0, function* () {
                if (msg.type === constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_INPUT_REQUEST) {
                    const predefinedAnswer = _.find(this.predefinedAnswers, a => msg.body.message.indexOf(a.searchString) >= 0);
                    if (!helpers_1.isInteractive() && !predefinedAnswer) {
                        this.$nsCloudErrorsService.fail(`Input is required but the process is not interactive. "${msg.body.message}"`);
                    }
                    let input = "";
                    if (predefinedAnswer) {
                        input = predefinedAnswer.answer;
                    }
                    else {
                        this.$logger.info();
                        if (msg.body.inputType === "password") {
                            input = yield this.$prompter.getPassword(msg.body.message);
                        }
                        else {
                            input = yield this.$prompter.getString(msg.body.message);
                        }
                    }
                    yield this.sendInput(msg, input);
                }
            });
            this.interactiveService.on(constants_1.CloudCommunicationEvents.MESSAGE, messageHandler);
            this.$nsCloudProcessService.attachToProcessExitSignals(this, () => {
                this.interactiveService.removeListener(constants_1.CloudCommunicationEvents.MESSAGE, messageHandler);
            });
            yield this.executeCore(args);
        });
    }
    sendInput(msg, input) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const inputBody = { inputType: msg.body.inputType, inputRequestId: msg.body.inputRequestId, content: input };
                const cloudMessage = { type: constants_1.CloudOperationMessageTypes.CLOUD_OPERATION_INPUT, cloudOperationId: msg.cloudOperationId, body: inputBody };
                yield this.interactiveService.sendCloudMessage(cloudMessage);
            }
            catch (err) {
                this.$logger.trace(`Can't send input to cloud operation ${msg.cloudOperationId}. Error is ${err}.`);
            }
        });
    }
}
exports.InteractiveCloudCommand = InteractiveCloudCommand;
