"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const url = require("url");
const path = require("path");
const constants_1 = require("./constants");
class HttpServer {
    constructor($logger, $fs) {
        this.$logger = $logger;
        this.$fs = $fs;
    }
    createServer(configuration) {
        if (!configuration.catchAll) {
            configuration.catchAll = (request, response) => {
                response.statusCode = 404;
                response.end();
            };
        }
        const server = http.createServer((request, response) => {
            const uriPath = url.parse(request.url).pathname;
            this.$logger.debug("Serving '%s'", uriPath);
            response.setHeader(constants_1.HTTP_HEADERS.CONNECTION, "close");
            if (!configuration.routes[uriPath]) {
                configuration.catchAll(request, response);
            }
            else {
                configuration.routes[uriPath](request, response);
            }
        });
        return server;
    }
    serveFile(fileName) {
        return (request, response) => {
            return new Promise((resolve, reject) => {
                const mimeTypes = {
                    ".html": constants_1.CONTENT_TYPES.TEXT_HTML,
                    ".jpeg": constants_1.CONTENT_TYPES.IMAGE_JPEG,
                    ".jpg": constants_1.CONTENT_TYPES.IMAGE_JPEG,
                    ".png": constants_1.CONTENT_TYPES.IMAGE_PNG,
                    ".js": constants_1.CONTENT_TYPES.TEXT_JAVASCRIPT,
                    ".css": constants_1.CONTENT_TYPES.TEXT_CSS
                };
                this.$logger.debug("Returning '%s'", fileName);
                const mimeType = mimeTypes[path.extname(fileName)];
                response.statusCode = constants_1.HTTP_STATUS_CODES.SUCCESS;
                response.setHeader(constants_1.HTTP_HEADERS.CONTENT_TYPE, mimeType);
                this.$fs.createReadStream(fileName).pipe(response);
                response.on("finish", () => {
                    resolve();
                });
            });
        };
    }
    redirect(response, targetUrl) {
        response.statusCode = 302;
        response.setHeader("Location", targetUrl);
        response.end();
    }
}
exports.HttpServer = HttpServer;
$injector.register("nsCloudHttpServer", HttpServer);
