"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class AccountsService {
    constructor($nsCloudErrorsService, $nsCloudPolicyService, $nsCloudServerAccountsService) {
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$nsCloudPolicyService = $nsCloudPolicyService;
        this.$nsCloudServerAccountsService = $nsCloudServerAccountsService;
    }
    getMyAccounts() {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            return this.$nsCloudServerAccountsService.getAccounts();
        });
    }
    getUsageInfo(accountIdOption) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const account = yield this.getAccountFromOption(accountIdOption);
            return this.$nsCloudServerAccountsService.getUsageInfo(account.id);
        });
    }
    getAccountFromOption(accountIdOption) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const accounts = yield this.getMyAccounts();
            if (!accountIdOption) {
                this.$nsCloudErrorsService.fail("Please provide accountId.");
            }
            let selectedAccount = _.find(accounts, a => a.id === accountIdOption);
            if (selectedAccount) {
                return selectedAccount;
            }
            else {
                const accountIndex = Number.parseInt(accountIdOption);
                selectedAccount = accounts[accountIndex - 1];
                if (!selectedAccount) {
                    this.$nsCloudErrorsService.fail("Invalid accountId index provided.");
                }
                return selectedAccount;
            }
        });
    }
    getAccountFeatures(accountIdOption) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const account = yield this.getAccountFromOption(accountIdOption);
            return this.$nsCloudServerAccountsService.getAccountFeatures(account.id);
        });
    }
    sendPoliciesToCloud() {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this.$nsCloudPolicyService.getNsCloudPoliciesSetting();
            return this.$nsCloudServerAccountsService.syncPolicies(data);
        });
    }
}
exports.AccountsService = AccountsService;
$injector.register("nsCloudAccountsService", AccountsService);
