"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
class ApplicationService {
    constructor($nsCloudPlatformService, $projectDataService, $devicesService, $projectChangesService, $staticConfig, $injector) {
        this.$nsCloudPlatformService = $nsCloudPlatformService;
        this.$projectDataService = $projectDataService;
        this.$devicesService = $devicesService;
        this.$projectChangesService = $projectChangesService;
        this.$staticConfig = $staticConfig;
        this.$injector = $injector;
        this.shouldUseOldCheckForChanges = true;
        const cliVersion = this.$staticConfig.version;
        this.shouldUseOldCheckForChanges = semver.valid(cliVersion) && semver.lt(cliVersion, semver.prerelease(cliVersion) ? "5.4.0-2019-05-16-13277" : "6.0.0");
    }
    shouldBuild(config) {
        return __awaiter(this, void 0, void 0, function* () {
            const projectData = this.$projectDataService.getProjectData(config.projectDir);
            config.release = config.release === undefined ? false : config.release;
            config.bundle = config.bundle === undefined ? false : config.bundle;
            if (this.shouldUseOldCheckForChanges) {
                yield this.$projectChangesService.checkForChanges({
                    platform: config.platform,
                    projectData,
                    projectChangesOptions: config
                });
            }
            else {
                const platformsDataService = this.$injector.resolve("platformsDataService");
                const platformData = platformsDataService.getPlatformData(config.platform, projectData);
                const prepareDataService = this.$injector.resolve("prepareDataService");
                const prepareData = prepareDataService.getPrepareData(projectData.projectDir, config.platform, config);
                yield this.$projectChangesService.checkForChanges(platformData, projectData, prepareData);
            }
            return this.$nsCloudPlatformService.shouldBuild(config, projectData);
        });
    }
    shouldInstall(config) {
        return __awaiter(this, void 0, void 0, function* () {
            const projectData = this.$projectDataService.getProjectData(config.projectDir);
            const device = yield this.$devicesService.getDevice(config.deviceIdentifier);
            return this.$nsCloudPlatformService.shouldInstall(config, projectData, device);
        });
    }
}
exports.ApplicationService = ApplicationService;
$injector.register("nsCloudApplicationService", ApplicationService);
