"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cloud_service_1 = require("./cloud-service");
class CloudAppleService extends cloud_service_1.CloudService {
    constructor($constants, $nsCloudErrorsService, $fs, $httpClient, $logger, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService, $nsCloudServerBuildService) {
        super($nsCloudErrorsService, $fs, $httpClient, $logger, $constants, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService);
        this.$nsCloudServerBuildService = $nsCloudServerBuildService;
    }
    get failedError() {
        return "Apple login failed.";
    }
    get failedToStartError() {
        return "Failed to start Apple login.";
    }
    appleLogin(credentials) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.executeCloudOperation("Apple login", (cloudOperationId) => __awaiter(this, void 0, void 0, function* () {
                const appleLoginData = { cloudOperationId, credentials };
                const response = yield this.$nsCloudServerBuildService.appleLogin(appleLoginData);
                this.$logger.trace("Apple login response", response);
                let appleLoginResult = yield this.waitForCloudOperationToFinish(appleLoginData.cloudOperationId, response, { silent: true });
                if (!appleLoginResult.data || !appleLoginResult.data.appleSessionBase64) {
                    this.$nsCloudErrorsService.fail(`Apple login failed. Reason is: ${appleLoginResult.errors}. Additional information: ${appleLoginResult.stderr || appleLoginResult.stdout}.`);
                }
                return appleLoginResult.data.appleSessionBase64;
            }));
            return result;
        });
    }
}
exports.CloudAppleService = CloudAppleService;
$injector.register("nsCloudAppleService", CloudAppleService);
