"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const uuid = require("uuid");
const querystring_1 = require("querystring");
const constants = require("../constants");
const cloud_service_1 = require("./cloud-service");
class CloudBuildService extends cloud_service_1.CloudService {
    constructor($constants, $nsCloudErrorsService, $fs, $httpClient, $logger, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService, $nsCloudBuildHelper, $nsCloudBuildPropertiesService, $mobileHelper, $devicePlatformsConstants, $nsCloudConfigurationService, $nsCloudAccountsService, $nsCloudServerBuildService, $nsCloudGitService, $nsCloudItmsServicesPlistHelper, $nsCloudUploadService, $nsCloudUserService, $nsCloudVersionService, $nsCloudEncryptionService, $nsCloudPlatformService, $projectHelper, $projectDataService, $qr, $nsCloudPlatformsData, $filesHashService) {
        super($nsCloudErrorsService, $fs, $httpClient, $logger, $constants, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService);
        this.$nsCloudBuildHelper = $nsCloudBuildHelper;
        this.$nsCloudBuildPropertiesService = $nsCloudBuildPropertiesService;
        this.$mobileHelper = $mobileHelper;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$nsCloudConfigurationService = $nsCloudConfigurationService;
        this.$nsCloudAccountsService = $nsCloudAccountsService;
        this.$nsCloudServerBuildService = $nsCloudServerBuildService;
        this.$nsCloudGitService = $nsCloudGitService;
        this.$nsCloudItmsServicesPlistHelper = $nsCloudItmsServicesPlistHelper;
        this.$nsCloudUploadService = $nsCloudUploadService;
        this.$nsCloudUserService = $nsCloudUserService;
        this.$nsCloudVersionService = $nsCloudVersionService;
        this.$nsCloudEncryptionService = $nsCloudEncryptionService;
        this.$nsCloudPlatformService = $nsCloudPlatformService;
        this.$projectHelper = $projectHelper;
        this.$projectDataService = $projectDataService;
        this.$qr = $qr;
        this.$nsCloudPlatformsData = $nsCloudPlatformsData;
        this.$filesHashService = $filesHashService;
    }
    get failedError() {
        return "Build failed.";
    }
    get failedToStartError() {
        return "Failed to start cloud build.";
    }
    getServerOperationOutputDirectory(options) {
        let result = path.join(options.projectDir, constants.CLOUD_TEMP_DIR_NAME, options.platform.toLowerCase());
        if (this.$mobileHelper.isiOSPlatform(options.platform)) {
            result = path.join(result, options.emulator ? constants.CLOUD_BUILD_DIRECTORY_NAMES.EMULATOR : constants.CLOUD_BUILD_DIRECTORY_NAMES.DEVICE);
        }
        return result;
    }
    getBuildOutputDirectory(options) {
        return this.getServerOperationOutputDirectory(options);
    }
    build(projectSettings, platform, buildConfiguration, accountId, androidBuildData, iOSBuildData, buildOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const result = yield this.executeCloudOperation("Cloud build", (cloudOperationId) => __awaiter(this, void 0, void 0, function* () {
                this.$logger.info("Getting accounts information...");
                const account = yield this.$nsCloudAccountsService.getAccountFromOption(accountId);
                this.$logger.info("Using account %s.", account.id);
                const buildResult = yield this.executeBuild(projectSettings, platform, buildConfiguration, cloudOperationId, account.id, androidBuildData, iOSBuildData, buildOptions);
                return buildResult;
            }));
            return result;
        });
    }
    executeBuild(projectSettings, platform, buildConfiguration, cloudOperationId, accountId, androidBuildData, iOSBuildData, buildOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const buildInformationString = `Cloud build of '${projectSettings.projectDir}', platform: '${platform}', ` +
                `configuration: '${buildConfiguration}'`;
            this.$logger.info(`${buildInformationString}.`);
            yield this.$nsCloudBuildPropertiesService.validateBuildProperties(platform, buildConfiguration, projectSettings.projectId, androidBuildData, iOSBuildData);
            if (buildOptions && buildOptions.shouldPrepare) {
                yield this.prepareProject(cloudOperationId, projectSettings, platform, buildConfiguration, iOSBuildData);
            }
            let buildFiles = [];
            const isReleaseBuild = this.$nsCloudBuildHelper.isReleaseConfiguration(buildConfiguration);
            const isAndroidBuild = this.$mobileHelper.isAndroidPlatform(platform);
            if (isAndroidBuild && isReleaseBuild) {
                buildFiles.push({
                    filename: uuid.v4(),
                    fullPath: androidBuildData.pathToCertificate,
                    disposition: constants.DISPOSITIONS.CRYPTO_STORE
                });
            }
            else if (this.$mobileHelper.isiOSPlatform(platform) && iOSBuildData.buildForDevice) {
                buildFiles.push({
                    filename: uuid.v4(),
                    fullPath: iOSBuildData.pathToCertificate,
                    disposition: constants.DISPOSITIONS.KEYCHAIN
                });
                const provisionData = this.$nsCloudBuildHelper.getMobileProvisionData(iOSBuildData.pathToProvision);
                buildFiles.push({
                    filename: `${provisionData.UUID}.mobileprovision`,
                    fullPath: iOSBuildData.pathToProvision,
                    disposition: constants.DISPOSITIONS.PROVISION
                });
            }
            const fileNames = buildFiles.map(buildFile => buildFile.filename);
            const buildCredentials = yield this.$nsCloudServerBuildService.getBuildCredentials({ appId: projectSettings.projectId, fileNames: fileNames });
            const filesToUpload = this.prepareFilesToUpload(buildCredentials.urls, buildFiles);
            const additionalCliFlags = [];
            if (projectSettings.bundle) {
                additionalCliFlags.push("--bundle");
            }
            if (projectSettings.useHotModuleReload && !isReleaseBuild) {
                additionalCliFlags.push("--hmr");
            }
            else {
                additionalCliFlags.push("--no-hmr");
            }
            if (!projectSettings.bundle && !projectSettings.useHotModuleReload) {
                additionalCliFlags.push("--no-bundle");
            }
            const useAabFlag = isAndroidBuild && androidBuildData && androidBuildData.aab;
            if (useAabFlag) {
                additionalCliFlags.push("--aab");
            }
            if (projectSettings.env) {
                const envOptions = _.map(projectSettings.env, (value, key) => `--env.${key}=${value}`);
                additionalCliFlags.push(...envOptions);
            }
            let buildProps = yield this.prepareBuildRequest({
                cloudOperationId: cloudOperationId,
                projectSettings,
                platform,
                buildConfiguration,
                buildCredentials,
                filesToUpload,
                additionalCliFlags,
                accountId
            });
            if (isAndroidBuild) {
                buildProps = yield this.$nsCloudBuildPropertiesService.getAndroidBuildProperties(projectSettings, buildProps, filesToUpload, androidBuildData);
            }
            else if (this.$mobileHelper.isiOSPlatform(platform)) {
                buildProps = yield this.$nsCloudBuildPropertiesService.getiOSBuildProperties(projectSettings, buildProps, filesToUpload, iOSBuildData);
            }
            this.emitStepChanged(cloudOperationId, constants.BUILD_STEP_NAME.BUILD, constants.BUILD_STEP_PROGRESS.START);
            const buildResponse = yield this.$nsCloudServerBuildService.startBuild(buildProps);
            this.$logger.trace("Build response:");
            this.$logger.trace(buildResponse);
            const buildResult = yield this.waitForCloudOperationToFinish(cloudOperationId, buildResponse, { silent: false });
            this.emitStepChanged(cloudOperationId, constants.BUILD_STEP_NAME.BUILD, constants.BUILD_STEP_PROGRESS.END);
            this.$logger.trace("Build result:");
            this.$logger.trace(buildResult);
            if (!buildResult.buildItems || !buildResult.buildItems.length) {
                this.$nsCloudErrorsService.fail(`Build failed. Reason is: ${buildResult.errors}. Additional information: ${buildResult.stderr}.`);
            }
            this.$logger.info(`Finished ${buildInformationString} successfully. Downloading result...`);
            const localBuildResult = yield this.downloadServerResult(cloudOperationId, buildResult, {
                projectDir: projectSettings.projectDir,
                platform,
                emulator: iOSBuildData && !iOSBuildData.buildForDevice,
                extension: useAabFlag ? "aab" : null
            });
            this.$logger.info(`The result of ${buildInformationString} successfully downloaded. OutputFilePath: ${localBuildResult}`);
            let qrData = null;
            if (!useAabFlag) {
                const buildResultUrl = this.getBuildResultUrl(buildResult);
                const itmsOptions = {
                    pathToProvision: iOSBuildData && iOSBuildData.pathToProvision,
                    projectId: projectSettings.projectId,
                    projectName: projectSettings.projectName,
                    url: buildResultUrl
                };
                qrData = {
                    originalUrl: buildResultUrl,
                    imageData: yield this.getImageData(buildResultUrl, itmsOptions)
                };
            }
            const result = {
                cloudOperationId: cloudOperationId,
                stderr: buildResult.stderr,
                stdout: buildResult.stdout,
                outputFilePath: localBuildResult,
                qrData
            };
            const platformData = this.$nsCloudPlatformsData.getPlatformData(platform, this.$projectDataService.getProjectData(projectSettings.projectDir));
            if (this.$filesHashService.saveHashesForProject && !projectSettings.useHotModuleReload) {
                yield this.$filesHashService.saveHashesForProject(platformData, path.dirname(localBuildResult));
            }
            const buildInfoFileDirname = path.dirname(result.outputFilePath);
            this.$nsCloudPlatformService.saveBuildInfoFile(projectSettings.projectDir, buildInfoFileDirname, platformData);
            return result;
        });
    }
    validateBuildProperties(platform, buildConfiguration, appId, androidBuildData, iOSBuildData) {
        global.showErrorForStoppedCloudBuilds();
        return this.$nsCloudBuildPropertiesService.validateBuildProperties(platform, buildConfiguration, appId, androidBuildData, iOSBuildData);
    }
    prepareFilesToUpload(amazonStorageEntries, buildFiles) {
        let result = [];
        _.each(amazonStorageEntries, amazonStorageEntry => {
            _.each(buildFiles, buildFile => {
                if (amazonStorageEntry.fileName === buildFile.filename) {
                    result.push(_.merge({ filePath: buildFile.fullPath, disposition: buildFile.disposition }, amazonStorageEntry));
                }
            });
        });
        return result;
    }
    prepareProject(cloudOperationId, projectSettings, platform, buildConfiguration, iOSBuildData) {
        return __awaiter(this, void 0, void 0, function* () {
            const projectData = this.$projectDataService.getProjectData(projectSettings.projectDir);
            let mobileProvisionData;
            let provision;
            if (iOSBuildData && iOSBuildData.pathToProvision) {
                mobileProvisionData = this.$nsCloudBuildHelper.getMobileProvisionData(iOSBuildData.pathToProvision);
                mobileProvisionData.Type = this.$nsCloudBuildHelper.getProvisionType(mobileProvisionData);
                provision = mobileProvisionData.UUID;
            }
            this.emitStepChanged(cloudOperationId, constants.BUILD_STEP_NAME.PREPARE, constants.BUILD_STEP_PROGRESS.START);
            const userAppGradleFilePath = path.join(projectData.appResourcesDirectoryPath, this.$devicePlatformsConstants.Android, "app.gradle");
            if (this.$fs.exists(userAppGradleFilePath)) {
                const appGradleContents = this.$fs.readText(userAppGradleFilePath);
                const appGradleReplacedContents = appGradleContents.replace(/__PACKAGE__/g, projectSettings.projectId);
                if (appGradleReplacedContents !== appGradleContents) {
                    this.$fs.writeFile(userAppGradleFilePath, appGradleReplacedContents);
                }
            }
            yield this.$nsCloudPlatformService.preparePlatform(projectSettings, platform, buildConfiguration, projectData, provision, mobileProvisionData);
            this.emitStepChanged(cloudOperationId, constants.BUILD_STEP_NAME.PREPARE, constants.BUILD_STEP_PROGRESS.END);
        });
    }
    prepareBuildRequest(settings) {
        return __awaiter(this, void 0, void 0, function* () {
            this.emitStepChanged(settings.cloudOperationId, constants.BUILD_STEP_NAME.UPLOAD, constants.BUILD_STEP_PROGRESS.START);
            let buildFiles;
            try {
                yield this.$nsCloudGitService.gitPushChanges(settings.projectSettings, { httpRemoteUrl: settings.buildCredentials.codeCommit.cloneUrlHttp }, settings.buildCredentials.codeCommit.credentials, { isNewRepository: settings.buildCredentials.codeCommit.isNewRepository });
                buildFiles = [
                    {
                        disposition: constants.DISPOSITIONS.PACKAGE_GIT,
                        sourceUri: settings.buildCredentials.codeCommitUrl
                    }
                ];
            }
            catch (err) {
                this.$logger.warn("Unable to use git, reason is:");
                this.$logger.warn(err.message);
                const filePath = yield this.$nsCloudBuildHelper.zipProject(settings.projectSettings.projectDir);
                const preSignedUrlData = yield this.$nsCloudServerBuildService.getPresignedUploadUrlObject(uuid.v4());
                const disposition = constants.DISPOSITIONS.PACKAGE_ZIP;
                settings.filesToUpload.push(_.merge({ filePath, disposition }, preSignedUrlData));
                buildFiles = [
                    {
                        disposition,
                        sourceUri: preSignedUrlData.publicDownloadUrl
                    }
                ];
            }
            for (const fileToUpload of settings.filesToUpload) {
                yield this.$nsCloudUploadService.uploadToS3(fileToUpload.filePath, fileToUpload.fileName, fileToUpload.uploadPreSignedUrl);
            }
            this.emitStepChanged(settings.cloudOperationId, constants.BUILD_STEP_NAME.UPLOAD, constants.BUILD_STEP_PROGRESS.END);
            const runtimeVersion = yield this.$nsCloudVersionService.getProjectRuntimeVersion(settings.projectSettings.projectDir, settings.platform);
            const cliVersion = yield this.$nsCloudVersionService.getCliVersion(runtimeVersion);
            const sanitizedProjectName = this.$projectHelper.sanitizeName(settings.projectSettings.projectName);
            const workflow = settings.projectSettings.workflowName && settings.projectSettings.workflowUrl && {
                workflowName: settings.projectSettings.workflowName,
                workflowUrl: settings.projectSettings.workflowUrl
            };
            const result = {
                cloudOperationId: settings.cloudOperationId,
                accountId: settings.accountId,
                properties: {
                    buildConfiguration: settings.buildConfiguration,
                    sharedCloud: settings.projectSettings.sharedCloud,
                    platform: settings.platform,
                    appIdentifier: settings.projectSettings.projectId,
                    frameworkVersion: cliVersion,
                    runtimeVersion: runtimeVersion,
                    sessionKey: settings.buildCredentials.sessionKey,
                    templateAppName: sanitizedProjectName,
                    projectName: sanitizedProjectName,
                    framework: "tns",
                    flavorId: settings.projectSettings.flavorId,
                    additionalCliFlags: settings.additionalCliFlags,
                    useIncrementalBuild: !settings.projectSettings.clean,
                    userEmail: this.$nsCloudUserService.getUser().email,
                    workspacePassword: yield this.$nsCloudEncryptionService.getWorkspacePassword(settings.projectSettings)
                },
                workflow,
                targets: [],
                buildFiles
            };
            const cloudConfigData = this.$nsCloudConfigurationService.getCloudConfigurationData();
            if (cloudConfigData && cloudConfigData.machineId) {
                result.machineId = cloudConfigData.machineId;
            }
            return result;
        });
    }
    getBuildResultUrl(buildResult) {
        return this.getServerResults(buildResult)[0].fullPath;
    }
    getServerResults(buildResult) {
        const result = _.find(buildResult.buildItems, b => b.disposition === constants.DISPOSITIONS.BUILD_RESULT);
        if (!result) {
            this.$nsCloudErrorsService.fail("No item with disposition BuildResult found in the build result items.");
        }
        return [result];
    }
    downloadServerResult(cloudOperationId, buildResult, buildOutputOptions) {
        const _super = Object.create(null, {
            downloadServerResults: { get: () => super.downloadServerResults }
        });
        return __awaiter(this, void 0, void 0, function* () {
            this.emitStepChanged(cloudOperationId, constants.BUILD_STEP_NAME.DOWNLOAD, constants.BUILD_STEP_PROGRESS.START);
            const targetFileNames = yield _super.downloadServerResults.call(this, buildResult, buildOutputOptions);
            this.emitStepChanged(cloudOperationId, constants.BUILD_STEP_NAME.DOWNLOAD, constants.BUILD_STEP_PROGRESS.END);
            return targetFileNames[0];
        });
    }
    getImageData(buildResultUrl, options) {
        return __awaiter(this, void 0, void 0, function* () {
            if (options.pathToProvision) {
                const provisionData = this.$nsCloudBuildHelper.getMobileProvisionData(options.pathToProvision);
                const provisionType = this.$nsCloudBuildHelper.getProvisionType(provisionData);
                if (provisionType !== constants.PROVISION_TYPES.ADHOC) {
                    return null;
                }
                const preSignedUrlData = yield this.$nsCloudServerBuildService.getPresignedUploadUrlObject(uuid.v4());
                yield this.$nsCloudUploadService.uploadToS3(this.$nsCloudItmsServicesPlistHelper.createPlistContent(options), preSignedUrlData.fileName, preSignedUrlData.uploadPreSignedUrl);
                return this.$qr.generateDataUri(`itms-services://?action=download-manifest&amp;url=${querystring_1.escape(preSignedUrlData.publicDownloadUrl)}`);
            }
            return this.$qr.generateDataUri(buildResultUrl);
        });
    }
    emitStepChanged(cloudOperationId, step, progress) {
        const buildStep = { cloudOperationId: cloudOperationId, step, progress };
        this.emit(constants.CLOUD_BUILD_EVENT_NAMES.STEP_CHANGED, buildStep);
    }
}
exports.CloudBuildService = CloudBuildService;
$injector.register("nsCloudBuildService", CloudBuildService);
