"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const constants = require("../constants");
const cloud_service_1 = require("./cloud-service");
const helpers_1 = require("../helpers");
class CloudCodesignService extends cloud_service_1.CloudService {
    constructor($constants, $nsCloudErrorsService, $fs, $httpClient, $logger, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService, $nsCloudServerBuildService, $projectHelper, $projectDataService) {
        super($nsCloudErrorsService, $fs, $httpClient, $logger, $constants, $nsCloudOperationFactory, $nsCloudOutputFilter, $nsCloudProcessService);
        this.$nsCloudServerBuildService = $nsCloudServerBuildService;
        this.$projectHelper = $projectHelper;
        this.$projectDataService = $projectDataService;
    }
    get failedError() {
        return "Generation of codesign files failed.";
    }
    get failedToStartError() {
        return "Failed to start generation of codesign files.";
    }
    generateCodesignFiles(codesignData, projectDir) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.executeCloudOperation("Cloud code sign", (cloudOperationId) => __awaiter(this, void 0, void 0, function* () {
                this.validateParameters(codesignData, projectDir);
                codesignData.clean = codesignData.clean === undefined ? true : codesignData.clean;
                const serverResult = yield this.executeGeneration(codesignData, projectDir, cloudOperationId);
                return serverResult;
            }));
            return result;
        });
    }
    getServerResults(codesignResult) {
        const result = _.filter(codesignResult.buildItems, b => b.disposition === constants.DISPOSITIONS.CERTIFICATE
            || b.disposition === constants.DISPOSITIONS.PROVISION);
        if (!result) {
            this.$nsCloudErrorsService.fail(`No item with disposition ${constants.DISPOSITIONS.CERTIFICATE} or ${constants.DISPOSITIONS.PROVISION} found in the server result items.`);
        }
        return result;
    }
    getServerOperationOutputDirectory(options) {
        return path.join(options.projectDir, constants.CODESIGN_FILES_DIR_NAME, options.platform.toLowerCase());
    }
    validateParameters(codesignData, projectDir) {
        if (!codesignData || !codesignData.username || !codesignData.password) {
            this.$nsCloudErrorsService.fail(`Codesign failed. Reason is missing code sign data. Apple Id and Apple Id password are required..`);
        }
        if (!projectDir) {
            this.$nsCloudErrorsService.fail(`Codesign failed. Reason is invalid project path.`);
        }
    }
    executeGeneration(codesignData, projectDir, cloudOperationId) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error("Auto Generation is currently not available. Please follow the GitHub issue for further information: https://github.com/NativeScript/sidekick-feedback/issues/435");
            const codesignInformationString = "generation of iOS certificate and provision files";
            this.$logger.info(`Starting ${codesignInformationString}.`);
            const projectData = this.$projectDataService.getProjectData(projectDir);
            const codesignRequest = yield this.prepareCodesignRequest(cloudOperationId, codesignData, projectData);
            const codesignResponse = yield this.$nsCloudServerBuildService.generateCodesignFiles(codesignRequest);
            this.$logger.trace(`Codesign response: ${JSON.stringify(codesignResponse)}`);
            const codesignResult = yield this.waitForCloudOperationToFinish(cloudOperationId, codesignResponse, { silent: true });
            this.$logger.trace("Codesign result:");
            this.$logger.trace(codesignResult);
            if (!codesignResult.buildItems || !codesignResult.buildItems.length) {
                let errText = codesignResult.errors || "";
                if (errText.indexOf("403 Forbidden") > -1) {
                    this.$logger.trace(`Codesign errors: ${errText}`);
                    errText = "The Code Signing Assistance service is temporary unavailable. Please try again later.";
                }
                const err = new Error(`Codesign failed. Reason is: ${errText}.`);
                err.stderr = codesignResult.stderr;
                throw err;
            }
            this.$logger.info(`Finished ${codesignInformationString} successfully. Downloading result...`);
            const localCodesignResults = yield this.downloadServerResults(codesignResult, {
                projectDir: projectData.projectDir,
                platform: codesignData.platform,
                emulator: false
            });
            this.$logger.info(`The result of ${codesignInformationString} successfully downloaded. Codesign files paths: ${localCodesignResults}`);
            const result = {
                cloudOperationId: cloudOperationId,
                stderr: codesignResult.stderr,
                stdout: codesignResult.stdout,
                fullOutput: codesignResult,
                outputFilesPaths: localCodesignResults
            };
            return result;
        });
    }
    prepareCodesignRequest(cloudOperationId, codesignData, projectData) {
        return __awaiter(this, void 0, void 0, function* () {
            const sanitizedProjectName = this.$projectHelper.sanitizeName(projectData.projectName);
            return {
                cloudOperationId: cloudOperationId,
                appId: helpers_1.getProjectId(projectData, codesignData.platform.toLowerCase()),
                appName: sanitizedProjectName,
                clean: codesignData.clean,
                username: codesignData.username,
                password: codesignData.password,
                sharedCloud: codesignData.sharedCloud,
                devices: codesignData.attachedDevices
            };
        });
    }
}
exports.CloudCodesignService = CloudCodesignService;
$injector.register("nsCloudCodesignService", CloudCodesignService);
