"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
class CloudEncryptionService {
    constructor($nsCloudEncryptionSettingsService, $nsCloudUserService, $nsCloudHashService) {
        this.$nsCloudEncryptionSettingsService = $nsCloudEncryptionSettingsService;
        this.$nsCloudUserService = $nsCloudUserService;
        this.$nsCloudHashService = $nsCloudHashService;
    }
    getWorkspacePassword(projectSettings) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const propertyName = this.$nsCloudHashService.getHash(`${this.$nsCloudUserService.getUser().email}_${projectSettings.projectId}`);
            const imageSettings = yield this.getNsCloudEncryptionSettings();
            let value = imageSettings[propertyName];
            if (!value) {
                value = this.generatePassword();
                yield this.setEncryptionSetting(propertyName, value);
            }
            return value;
        });
    }
    generatePassword() {
        return helpers_1.getRandomString(CloudEncryptionService.IMAGE_PASSWORD_LENGTH);
    }
    getNsCloudEncryptionSettings() {
        return __awaiter(this, void 0, void 0, function* () {
            const settings = (yield this.$nsCloudEncryptionSettingsService.getSettingValue(CloudEncryptionService.NS_CLOUD_ENCRYPTION_SETTINGS)) || {};
            return settings;
        });
    }
    setEncryptionSetting(prop, value) {
        return __awaiter(this, void 0, void 0, function* () {
            const imageSettings = yield this.getNsCloudEncryptionSettings();
            imageSettings[prop] = value;
            yield this.$nsCloudEncryptionSettingsService.saveSetting(CloudEncryptionService.NS_CLOUD_ENCRYPTION_SETTINGS, imageSettings);
        });
    }
}
CloudEncryptionService.NS_CLOUD_ENCRYPTION_SETTINGS = "nsCloudEncryptionSettings";
CloudEncryptionService.IMAGE_PASSWORD_LENGTH = 64;
exports.CloudEncryptionService = CloudEncryptionService;
$injector.register("nsCloudEncryptionService", CloudEncryptionService);
