"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
const constants = require("../constants");
class CloudPlatformService {
    constructor($injector, $staticConfig) {
        this.$injector = $injector;
        this.$staticConfig = $staticConfig;
        const cliVersion = this.$staticConfig.version;
        this.shouldUseOldPlatformService = semver.valid(cliVersion) && semver.lt(cliVersion, semver.prerelease(cliVersion) ? "5.4.0-2019-05-16-13277" : "6.0.0");
    }
    get $nsCloudBuildCommandHelper() {
        return this.$injector.resolve("nsCloudBuildCommandHelper");
    }
    shouldBuild(config, projectData) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            if (this.shouldUseOldPlatformService) {
                const $platformService = this.$injector.resolve("platformService");
                return $platformService.shouldBuild(config.platform, projectData, config, config.outputPath);
            }
            const $buildController = this.$injector.resolve("buildController");
            return $buildController.shouldBuild(config);
        });
    }
    shouldInstall(config, projectData, device) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            if (this.shouldUseOldPlatformService) {
                const $platformService = this.$injector.resolve("platformService");
                return $platformService.shouldInstall(device, projectData, config, config.outputPath);
            }
            const $deviceInstallAppService = this.$injector.resolve("deviceInstallAppService");
            return $deviceInstallAppService.shouldInstall(device, config);
        });
    }
    deployPlatform(platform, outputDirectoryPath) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            const $deployCommandHelper = this.$injector.resolve("deployCommandHelper");
            if (this.shouldUseOldPlatformService) {
                const deployPlatformInfo = $deployCommandHelper.getDeployPlatformInfo(platform);
                deployPlatformInfo.buildPlatform = this.$nsCloudBuildCommandHelper.buildPlatform.bind(this.$nsCloudBuildCommandHelper);
                deployPlatformInfo.outputPath = outputDirectoryPath;
                deployPlatformInfo.nativePrepare = { skipNativePrepare: true };
                const platformService = this.$injector.resolve("platformService");
                return platformService.deployPlatform(deployPlatformInfo);
            }
            return $deployCommandHelper.deploy(platform, {
                getOutputDirectory: () => outputDirectoryPath,
                buildPlatform: this.$nsCloudBuildCommandHelper.buildPlatform.bind(this.$nsCloudBuildCommandHelper),
                skipNativePrepare: true
            });
        });
    }
    saveBuildInfoFile(projectDir, buildInfoFileDirname, platformData) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            if (this.shouldUseOldPlatformService) {
                const $platformService = this.$injector.resolve("platformService");
                return $platformService.saveBuildInfoFile(platformData.normalizedPlatformName.toLowerCase(), projectDir, buildInfoFileDirname);
            }
            const $buildInfoFileService = this.$injector.resolve("buildInfoFileService");
            return $buildInfoFileService.saveLocalBuildInfo(platformData, buildInfoFileDirname);
        });
    }
    preparePlatform(projectSettings, platform, buildConfiguration, projectData, provision, mobileProvisionData) {
        return __awaiter(this, void 0, void 0, function* () {
            global.showErrorForStoppedCloudBuilds();
            if (this.shouldUseOldPlatformService) {
                const appFilesUpdaterOptions = {
                    bundle: projectSettings.bundle,
                    useHotModuleReload: projectSettings.useHotModuleReload,
                    release: buildConfiguration && buildConfiguration.toLowerCase() === constants.CLOUD_BUILD_CONFIGURATIONS.RELEASE.toLowerCase()
                };
                const config = {
                    provision,
                    mobileProvisionData,
                    sdk: null,
                    frameworkPath: null,
                    ignoreScripts: false,
                    teamId: undefined
                };
                const $platformService = this.$injector.resolve("platformService");
                return $platformService.preparePlatform({
                    platform,
                    appFilesUpdaterOptions,
                    projectData,
                    config,
                    filesToSync: [],
                    nativePrepare: { skipNativePrepare: true },
                    platformTemplate: null,
                    env: projectSettings.env
                });
            }
            const $prepareController = this.$injector.resolve("prepareController");
            return $prepareController.prepare({
                projectDir: projectData.projectDir,
                platform,
                useHotModuleReload: projectSettings.useHotModuleReload,
                release: buildConfiguration && buildConfiguration.toLowerCase() === constants.CLOUD_BUILD_CONFIGURATIONS.RELEASE.toLowerCase(),
                provision,
                mobileProvisionData,
                teamId: null,
                env: projectSettings.env,
                nativePrepare: { skipNativePrepare: true }
            });
        });
    }
}
exports.CloudPlatformService = CloudPlatformService;
$injector.register("nsCloudPlatformService", CloudPlatformService);
