"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const os_1 = require("os");
const constants_1 = require("../constants");
class CloudServicesPolicyService {
    constructor($fs, $nsCloudPolicyService, $nsCloudHashService) {
        this.$fs = $fs;
        this.$nsCloudPolicyService = $nsCloudPolicyService;
        this.$nsCloudHashService = $nsCloudHashService;
    }
    acceptCloudServicesPolicy() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.$nsCloudPolicyService.accept({ policyName: constants_1.Policies.CLOUD_SERVICES_POLICY_NAME, content: yield this.getCloudServicesFullMessageHash() });
        });
    }
    shouldAcceptCloudServicesPolicy() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.$nsCloudPolicyService.shouldAcceptPolicy({ policyName: constants_1.Policies.CLOUD_SERVICES_POLICY_NAME, content: yield this.getCloudServicesFullMessageHash() });
        });
    }
    getCloudServicesFullMessage() {
        return __awaiter(this, void 0, void 0, function* () {
            const policyData = yield this.getCloudServicesPolicyData();
            const message = _.map(policyData.reasonsForCollectingPersonalData, r => `• ${r}`);
            message.unshift(policyData.collectedPersonalDataMessage + ":");
            message.push("You can review the Progress Software Privacy Policy at " + policyData.privacyPolicyUrl.yellow);
            message.push("You can submit a GDPR Data Subject Access Request at " + policyData.personalDataSubjectAccessRequestUrl.yellow);
            return message.join(os_1.EOL);
        });
    }
    getCloudServicesPolicyData() {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                collectedPersonalDataMessage: yield this.getLocalPolicyContent(CloudServicesPolicyService.CLOUD_SERVICES_POLICY_FILE_NAME),
                reasonsForCollectingPersonalData: yield this.getPersonalDataCollectionReasons(),
                personalDataSubjectAccessRequestUrl: CloudServicesPolicyService.PERSONAL_DATA_SUBJECT_ACCESS_REQUEST_URL,
                privacyPolicyUrl: CloudServicesPolicyService.PRIVACY_POLICY_URL
            };
        });
    }
    getCloudServicesFullMessageHash() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.$nsCloudHashService.getHash(yield this.getCloudServicesFullMessage());
        });
    }
    getPersonalDataCollectionReasons() {
        return __awaiter(this, void 0, void 0, function* () {
            const text = yield this.getLocalPolicyContent(CloudServicesPolicyService.REASONS_FOR_COLLECTING_PERSONAL_DATA_FILE_NAME);
            return text.split(os_1.EOL).filter(r => !!r);
        });
    }
    getLocalPolicyContent(policy) {
        return __awaiter(this, void 0, void 0, function* () {
            const policyPath = path_1.join(__dirname, "..", "..", "resources", CloudServicesPolicyService.POLICIES, policy);
            return yield this.$fs.readText(policyPath);
        });
    }
}
CloudServicesPolicyService.POLICIES = "policies";
CloudServicesPolicyService.CLOUD_SERVICES_POLICY_FILE_NAME = `${constants_1.Policies.CLOUD_SERVICES_POLICY_NAME}.txt`;
CloudServicesPolicyService.REASONS_FOR_COLLECTING_PERSONAL_DATA_FILE_NAME = "personal-data-collection-reasons.txt";
CloudServicesPolicyService.PERSONAL_DATA_SUBJECT_ACCESS_REQUEST_URL = "https://app.onetrust.com/app/#/webform/7897e80a-b8a4-4797-883a-bdacfe1ab8e4";
CloudServicesPolicyService.PRIVACY_POLICY_URL = "https://www.progress.com/legal/privacy-policy";
exports.CloudServicesPolicyService = CloudServicesPolicyService;
$injector.register("nsCloudServicesPolicyService", CloudServicesPolicyService);
