"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../constants");
class KinveyService {
    constructor($nsCloudKinveyRequestService) {
        this.$nsCloudKinveyRequestService = $nsCloudKinveyRequestService;
    }
    getApps() {
        return this.$nsCloudKinveyRequestService.getApps();
    }
    createApp(input) {
        return this.$nsCloudKinveyRequestService.createApp(input);
    }
    createAuthService(input) {
        return __awaiter(this, void 0, void 0, function* () {
            input.identityStoreOptions = input.identityStoreOptions || Object.create(null);
            const app = yield this.$nsCloudKinveyRequestService.getApp(input.appId);
            const createIdentityStoreInput = {
                name: input.identityStoreOptions.name || this.getIdentityStoreName(app.name, input.authServiceOptions.provider.type, input.environmentId),
                access: input.identityStoreOptions.access || {
                    writers: {
                        environments: [input.environmentId]
                    }
                }
            };
            const identityStore = yield this.$nsCloudKinveyRequestService.createIdentityStore(createIdentityStoreInput);
            const createAuthServiceInput = this.getCreateAuthServiceInput(app.name, identityStore.id, input.redirectUri, input.authServiceOptions);
            const authService = yield this.$nsCloudKinveyRequestService.createAuthService(createAuthServiceInput);
            const env = yield this.$nsCloudKinveyRequestService.getEnvironment(input.environmentId);
            env.identityStoreId = identityStore.id;
            env.defaultAuthServiceId = authService.id;
            yield this.$nsCloudKinveyRequestService.updateEnvironment(env);
            yield this.changeDefaultAuthService({ environmentId: input.environmentId, authServiceId: authService.id, identityStoreId: identityStore.id });
            return authService;
        });
    }
    updateAuthService(input) {
        if (_.isEmpty(input.authService.customizedLogin)) {
            delete input.authService.customizedLogin;
        }
        if (_.isEmpty(input.authService.provider.authentication)) {
            delete input.authService.provider.authentication;
        }
        return this.$nsCloudKinveyRequestService.updateAuthService(input.authServiceId, input.authService);
    }
    getAuthServices(input) {
        return __awaiter(this, void 0, void 0, function* () {
            const identityStores = yield this.$nsCloudKinveyRequestService.getIdentityStores();
            const identityStoresForEnvironment = _.filter(identityStores, i => i.access.writers.environments.indexOf(input.environmentId) >= 0);
            const result = {};
            for (let is of identityStoresForEnvironment) {
                result[is.id] = yield this.$nsCloudKinveyRequestService.getIdentityStoreAuthServices(is.id);
            }
            return result;
        });
    }
    getDefaultAuthService(input) {
        return __awaiter(this, void 0, void 0, function* () {
            const env = yield this.$nsCloudKinveyRequestService.getEnvironment(input.environmentId);
            if (!env.defaultAuthServiceId) {
                return null;
            }
            try {
                const result = yield this.$nsCloudKinveyRequestService.getAuthService(env.defaultAuthServiceId);
                return result;
            }
            catch (err) {
                if (err && err.response && err.response.statusCode === 404) {
                    return null;
                }
                throw err;
            }
        });
    }
    changeDefaultAuthService(input) {
        return __awaiter(this, void 0, void 0, function* () {
            const env = yield this.$nsCloudKinveyRequestService.getEnvironment(input.environmentId);
            env.identityStoreId = input.identityStoreId;
            env.defaultAuthServiceId = input.authServiceId;
            yield this.$nsCloudKinveyRequestService.updateEnvironment(env);
            return this.getDefaultAuthService({ environmentId: input.environmentId });
        });
    }
    getCreateAuthServiceInput(appName, isId, redirectUri, createAuthServiceOptions) {
        const result = {
            name: createAuthServiceOptions.name || this.getAuthServiceName(appName, createAuthServiceOptions.provider.type, isId),
            identityStoreId: createAuthServiceOptions.identityStoreId || isId,
            redirectUri: createAuthServiceOptions.redirectUri || redirectUri || ["http://example.com"],
            grantTtl: createAuthServiceOptions.grantTtl || 30,
            tokenTtl: createAuthServiceOptions.tokenTtl || 3600,
            refresh: _.has(createAuthServiceOptions, "refresh") ? createAuthServiceOptions.refresh : true,
            refreshTtl: createAuthServiceOptions.refreshTtl || 120960,
            customizedLogin: createAuthServiceOptions.customizedLogin,
            description: createAuthServiceOptions.description || `${appName} auth service`,
            access: createAuthServiceOptions.access,
            provider: this.getAuthProviderInput(createAuthServiceOptions.provider)
        };
        return result;
    }
    getAuthProviderInput(provider) {
        const result = {
            allowedAttributes: provider.allowedAttributes || ["id", "audience"],
            authentication: provider.authentication,
            datalinkHeaderMapping: provider.datalinkHeaderMapping || { "client_token": "X-Kinvey-Auth" },
            remoteService: provider.remoteService,
            type: provider.type,
            uniqueUserContextField: provider.uniqueUserContextField,
            options: this.getAuthProviderOptions(provider.type, provider.options)
        };
        return result;
    }
    getAuthProviderOptions(providerType, options) {
        if (providerType === constants_1.Authentication.OAuth2) {
            const res = options;
            res.grantType = res.grantType || "authorization-code";
            res.includeClientIdInTokenRequest = _.has(res, "includeClientIdInTokenRequest") ? res.includeClientIdInTokenRequest : false;
            res.includeClientSecretInTokenRequest = _.has(res, "includeClientSecretInTokenRequest") ? res.includeClientSecretInTokenRequest : false;
            return res;
        }
        return options;
    }
    getAuthServiceName(appName, authType, isId) {
        return `${appName}-${isId}-${authType}-auth-service`;
    }
    getIdentityStoreName(appName, authType, envId) {
        return `${appName}-${envId}-${authType}-identity-store`;
    }
}
exports.KinveyService = KinveyService;
$injector.register("nsCloudKinveyService", KinveyService);
