"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("../helpers");
class PolicyService {
    constructor($nsCloudErrorsService, $fs, $httpClient, $nsCloudHashService, $userSettingsService, $nsCloudTempService) {
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$fs = $fs;
        this.$httpClient = $httpClient;
        this.$nsCloudHashService = $nsCloudHashService;
        this.$userSettingsService = $userSettingsService;
        this.$nsCloudTempService = $nsCloudTempService;
    }
    shouldAcceptPolicy(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldAskToAccept = yield this.shouldAskToAcceptPolicy(data.policyName);
            if (shouldAskToAccept) {
                return true;
            }
            const currentContent = yield this.getPolicyUserSetting(data.policyName);
            const acceptableContent = yield this.getPolicyContentFromAcceptData(data);
            return currentContent !== acceptableContent;
        });
    }
    shouldAskToAcceptPolicy(policy) {
        return __awaiter(this, void 0, void 0, function* () {
            const currentContent = yield this.getPolicyUserSetting(policy);
            return !currentContent;
        });
    }
    accept(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const policyCloudContent = yield this.getPolicyContentFromAcceptData(data);
            if (!policyCloudContent) {
                this.$nsCloudErrorsService.fail("Invalid policy.");
            }
            yield this.setPolicyUserSetting(data.policyName, policyCloudContent);
        });
    }
    getPolicyUserSetting(policy) {
        return __awaiter(this, void 0, void 0, function* () {
            const policiesKey = yield this.getNsCloudPoliciesSetting();
            return policiesKey ? policiesKey[policy] : null;
        });
    }
    getNsCloudPoliciesSetting() {
        return __awaiter(this, void 0, void 0, function* () {
            const setting = (yield this.$userSettingsService.getSettingValue(PolicyService.NS_CLOUD_POLICIES)) || {};
            return setting;
        });
    }
    setPolicyUserSetting(policy, value) {
        return __awaiter(this, void 0, void 0, function* () {
            const policiesKey = yield this.getNsCloudPoliciesSetting();
            policiesKey[policy] = value;
            yield this.$userSettingsService.saveSetting(PolicyService.NS_CLOUD_POLICIES, policiesKey);
        });
    }
    getPolicyContentFromAcceptData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (data.content) {
                return data.content;
            }
            else if (data.policyUri) {
                let fileHash = data.policyUri;
                if (helpers_1.isUrl(data.policyUri)) {
                    const tempPolicyFile = yield this.$nsCloudTempService.path({ prefix: data.policyName, suffix: ".txt" });
                    yield this.$httpClient.httpRequest({
                        url: data.policyUri,
                        pipeTo: this.$fs.createWriteStream(tempPolicyFile)
                    });
                    fileHash = tempPolicyFile;
                }
                return yield this.$nsCloudHashService.getLocalFileHash(fileHash);
            }
            return null;
        });
    }
}
PolicyService.NS_CLOUD_POLICIES = "nsCloudPolicies";
exports.PolicyService = PolicyService;
$injector.register("nsCloudPolicyService", PolicyService);
