"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../constants");
const server_service_base_1 = require("./server-service-base");
class ServerAuthService extends server_service_base_1.ServerServiceBase {
    constructor($nsCloudConfigManager, $nsCloudServerServicesProxy, $nsAccountUtils, $nsCloudUserService, $injector) {
        super($nsCloudServerServicesProxy, $injector);
        this.$nsCloudConfigManager = $nsCloudConfigManager;
        this.$nsCloudServerServicesProxy = $nsCloudServerServicesProxy;
        this.$nsAccountUtils = $nsAccountUtils;
        this.$nsCloudUserService = $nsCloudUserService;
        this.serviceName = constants_1.AUTH_SERVICE_NAME;
    }
    getLoginUrl(port) {
        const loginUrl = this.getAuthUrl("api/login");
        return `${loginUrl}?port=${port}`;
    }
    refreshToken(refreshToken) {
        return this.sendRequest(constants_1.HTTP_METHODS.POST, "api/refresh-token", { refreshToken });
    }
    devLogin(username, password, instanceId) {
        const headers = Object.create(null);
        if (this.$nsAccountUtils.isKinveyUser()) {
            this.$nsCloudUserService.clearUserData();
            headers[constants_1.HTTP_HEADERS.X_NS_INSTANCE_ID] = instanceId;
        }
        return this.sendRequest(constants_1.HTTP_METHODS.POST, "api/credentials-grant", { username, password }, headers);
    }
    getTokenState(token) {
        return this.sendRequest(constants_1.HTTP_METHODS.POST, "api/token-state", { token });
    }
    getLogoutUrl() {
        return this.getAuthUrl("api/logout");
    }
    getAuthUrl(urlPath) {
        const proto = this.$nsCloudServerServicesProxy.getServiceProto(constants_1.AUTH_SERVICE_NAME);
        const host = this.$nsCloudConfigManager.getServiceDomainName(constants_1.AUTH_SERVICE_NAME);
        const url = this.$nsCloudServerServicesProxy.getUrlPath(constants_1.AUTH_SERVICE_NAME, urlPath);
        return `${proto}://${host}${url}`;
    }
}
exports.ServerAuthService = ServerAuthService;
$injector.register("nsCloudServerAuthService", ServerAuthService);
