"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../constants");
class ServerRequestService {
    constructor($nsCloudAuthenticationService, $logger, $injector) {
        this.$nsCloudAuthenticationService = $nsCloudAuthenticationService;
        this.$logger = $logger;
        this.$injector = $injector;
    }
    call(options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.proxyService.call(options);
            }
            catch (requestError) {
                if (requestError.response && requestError.response.statusCode === constants_1.HTTP_STATUS_CODES.UNAUTHORIZED) {
                    this.$logger.trace("Access token expired. Trying to issue a new one.");
                    try {
                        yield this.$nsCloudAuthenticationService.refreshCurrentUserToken();
                        return this.proxyService.call(options);
                    }
                    catch (refreshTokenError) {
                        this.$logger.trace("Cannot issue new access token. Reason is:");
                        this.$logger.trace(refreshTokenError);
                        const error = new Error("User is not logged in or the access token and the refresh tokens are expired.");
                        error.code = constants_1.HTTP_STATUS_CODES.UNAUTHORIZED;
                        throw error;
                    }
                }
                else {
                    throw requestError;
                }
            }
        });
    }
    get proxyService() {
        return this.$injector.resolve("nsCloudServerServicesProxy");
    }
}
exports.ServerRequestService = ServerRequestService;
$injector.register("nsCloudServerRequestService", ServerRequestService);
