"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../../constants");
class ServerServiceBase {
    constructor(requestService, $injector) {
        this.requestService = requestService;
        this.$injector = $injector;
    }
    get $nsCloudErrorsService() {
        return this.$injector.resolve("errors");
    }
    get $nsCloudEulaService() {
        return this.$injector.resolve("nsCloudEulaService");
    }
    sendRequest(method, urlPath, body, headers, resultStream) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ensureEulaIsAccepted();
            const requestOptions = {
                serviceName: this.serviceName,
                method: method,
                urlPath: urlPath,
                bodyValues: this.createJsonBody(body),
                accept: constants_1.CONTENT_TYPES.APPLICATION_JSON,
                headers: headers,
                resultStream: resultStream
            };
            return this.requestService.call(requestOptions);
        });
    }
    ensureEulaIsAccepted() {
        return __awaiter(this, void 0, void 0, function* () {
            const eulaData = yield this.$nsCloudEulaService.getEulaDataWithCache();
            if (eulaData.shouldAcceptEula) {
                this.$nsCloudErrorsService.fail(`EULA is not accepted, cannot use cloud services.`);
            }
        });
    }
    createJsonBody(body) {
        if (!body) {
            return null;
        }
        if (typeof (body) !== "string") {
            body = JSON.stringify(body);
        }
        return [{
                contentType: constants_1.CONTENT_TYPES.APPLICATION_JSON,
                value: body,
                name: "body"
            }];
    }
}
exports.ServerServiceBase = ServerServiceBase;
