"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const constants_1 = require("../constants");
class UploadService {
    constructor($nsCloudServerBuildService, $httpClient, $nsCloudErrorsService, $fs) {
        this.$nsCloudServerBuildService = $nsCloudServerBuildService;
        this.$httpClient = $httpClient;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
        this.$fs = $fs;
        this.serviceName = constants_1.BUILD_SERVICE_NAME;
    }
    uploadToS3(filePathOrContent, fileNameInS3, uploadPreSignedUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            fileNameInS3 = fileNameInS3 || uuid.v4();
            let preSignedUrlData;
            if (!uploadPreSignedUrl) {
                preSignedUrlData = yield this.$nsCloudServerBuildService.getPresignedUploadUrlObject(fileNameInS3);
                uploadPreSignedUrl = preSignedUrlData.uploadPreSignedUrl;
            }
            const requestOpts = {
                url: uploadPreSignedUrl,
                method: constants_1.HTTP_METHODS.PUT
            };
            if (this.$fs.exists(filePathOrContent)) {
                requestOpts.body = this.$fs.readFile(filePathOrContent);
            }
            else {
                requestOpts.body = filePathOrContent;
            }
            requestOpts.headers = requestOpts.headers || {};
            requestOpts.headers["Content-Length"] = requestOpts.body.length;
            try {
                yield this.$httpClient.httpRequest(requestOpts);
            }
            catch (err) {
                this.$nsCloudErrorsService.failWithHelp({ formatStr: `Error while uploading ${filePathOrContent} to S3. Errors is: ${err.message}`, proxyAuthenticationRequired: err.proxyAuthenticationRequired, suppressCommandHelp: true });
            }
            return preSignedUrlData && preSignedUrlData.publicDownloadUrl;
        });
    }
}
exports.UploadService = UploadService;
$injector.register("nsCloudUploadService", UploadService);
