"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
class UserServiceBase {
    constructor($injector, $logger, $fs, $nsCloudErrorsService) {
        this.$injector = $injector;
        this.$logger = $logger;
        this.$fs = $fs;
        this.$nsCloudErrorsService = $nsCloudErrorsService;
    }
    get $nsCloudServerAccountsService() {
        return this.$injector.resolve("nsCloudServerAccountsService");
    }
    hasUser() {
        try {
            const user = this.getUser();
            return !!user;
        }
        catch (err) {
            return false;
        }
    }
    getUser() {
        const userData = this.getUserData();
        return userData.userInfo;
    }
    getUserData() {
        try {
            return this.$fs.readJson(this.userFilePath);
        }
        catch (err) {
            this.$logger.trace("Error while getting current user info:");
            this.$logger.trace(err);
            this.$nsCloudErrorsService.fail("Not logged in.");
        }
    }
    setToken(token) {
        if (token) {
            const newUserData = _.extend(this.getUserData(), token);
            this.setUserData(newUserData);
        }
        else {
            this.clearUserData();
        }
    }
    setUserData(userData) {
        if (userData) {
            this.$fs.writeJson(this.userFilePath, userData);
        }
        else {
            this.$fs.deleteFile(this.userFilePath);
        }
    }
    clearUserData() {
        this.$fs.deleteFile(this.userFilePath);
    }
    getUserAvatar() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.hasUser()) {
                return null;
            }
            const userInfo = yield this.$nsCloudServerAccountsService.getUserInfo();
            return userInfo.externalAvatarUrl;
        });
    }
}
exports.UserServiceBase = UserServiceBase;
